# based on Robert Collins code

"""Show all 'heads' in a repository"""


from bzrlib.commands import Command, display_command, register_command


class cmd_heads(Command):
    """Show all revisions in a repository not having descendants.
    """
    encoding_type = "replace"

    @display_command
    def run(self):
        from bzrlib import errors
        from bzrlib.osutils import format_date
        import bzrlib.repository

        to_file = self.outf

        try:
            r = bzrlib.branch.Branch.open_containing('.')[0].repository
        except errors.NotBranchError:
            try:
                r = bzrlib.repository.Repository.open('.')
            except errors.NotBranchError:
                print >>to_file, \
                      ("You need to run this command "
                       "either from the root of a shared repository,\n"
                       "or from a branch.")
                return 3

        g = r.get_revision_graph()
        possible_heads = set(g.keys())
        not_heads = set()
        for parents in g.values():
            not_heads.update(set(parents))
        
        heads = possible_heads.difference(not_heads)

        # TODO: use different sorting schemes instead of alphabetical sort
        heads = list(heads)
        heads.sort()
        
        indent = ' '*2
        show_timezone = 'original'
        
        for head in heads:
            print >>to_file, "revision-id:", head
            rev = r.get_revision(head)
            # borrowed from LongLogFormatter
            print >>to_file,  indent+'committer:', rev.committer
            try:
                print >>to_file, indent+'branch nick: %s' % \
                    rev.properties['branch-nick']
            except KeyError:
                pass
            date_str = format_date(rev.timestamp,
                                   rev.timezone or 0,
                                   show_timezone)
            print >>to_file,  indent+'timestamp: %s' % date_str
    
            print >>to_file,  indent+'message:'
            if not rev.message:
                print >>to_file,  indent+'  (no message)'
            else:
                message = rev.message.rstrip('\r\n')
                for l in message.split('\n'):
                    print >>to_file,  indent+'  ' + l
            print

        if not heads:
            print >>to_file, 'No heads found'
            return 1
#/class cmd_heads


register_command(cmd_heads)
